import 'package:flutter/material.dart';
import 'package:og1_dashboard/og1_dashboard.dart';
import 'package:og1_dashboard/src/common/models/page_layout.dart';
import 'package:url_launcher/url_launcher.dart';

class PageGrid extends StatelessWidget {
  final List<AppIcon> appIcons;
  final PageLayout pageLayout;

  const PageGrid({Key? key, required this.appIcons, required this.pageLayout})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SafeArea(
        child: Column(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: _generateTableRows(appIcons, pageLayout)));
  }

  List<Row> _generateTableRows(List<AppIcon> appIcons, PageLayout pageLayout) {
    List<Row> tableRows = [];
    List<SizedBox> tableRowCells = [];

    for (int i = 0; i < pageLayout.appsPerPage; i++) {
      if (i % pageLayout.crossAxisCount == 0) {
        tableRowCells = [];
      }

      i < appIcons.length
          ? tableRowCells.add(_generateTableCell(appIcons[i]))
          : tableRowCells.add(_generateEmptyTableCell());

      if (i % pageLayout.crossAxisCount == 0) {
        Row tableRow = Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: tableRowCells);
        tableRows.add(tableRow);
      }
    }
    return tableRows;
  }

  SizedBox _generateTableCell(AppIcon appIcon) {
    return SizedBox(
      width: pageLayout.iconWidth + 10,
      child: Column(mainAxisAlignment: MainAxisAlignment.center, children: [
        GestureDetector(
            onTap: appIcon.onTap ?? () => _launchInApp(appIcon.url),
            child: ClipRRect(
                borderRadius: BorderRadius.circular(8.0),
                child: Image.asset(
                  appIcon.icon,
                  width: pageLayout.iconWidth,
                  height: pageLayout.iconHeight,
                  fit: BoxFit.scaleDown,
                  alignment: FractionalOffset.center,
                ))),
        Padding(
            padding: const EdgeInsets.only(top: 5),
            child: RichText(
              overflow: TextOverflow.clip,
              text: TextSpan(
                  text: appIcon.title, style: const TextStyle(fontSize: 12)),
              maxLines: 1,
            )),
      ]),
    );
  }

  SizedBox _generateEmptyTableCell() => SizedBox(
        width: pageLayout.iconWidth + 10,
        height: pageLayout.iconHeight + 10,
      );

  Future<void> _launchInApp(String url) async {
    if (await canLaunch(url)) {
      await launch(
        url,
        forceSafariVC: true,
        forceWebView: true,
        enableJavaScript: true,
        enableDomStorage: true,
        headers: <String, String>{'my_header_key': 'my_header_value'},
      );
    } else {
      throw 'Could not launch $url';
    }
  }
}
